'use strict';

const debug = require('debug')('eko:EkoQueueDispatcher/JobInstance');
const path = require('path');
const co = require('co');

/**
 * Find job class, instantiate and return
 * @param  {string} job
 * @param  {string} jobsPath
 * @return {object|boolean}
 */
const instantiate = (job, jobsPath) => {
  try {
    const jobModule = path.join(jobsPath, `${job}-job`);
    const JobClass = require(jobModule);

    return new JobClass;
  } catch (error) {
    debug(`No job class found for ${job}`);

    return false;
  }
};

/**
 * Constructor method for job instance
 * @param  {string} job
 * @param  {string} jobsPath
 * @return {object}
 */
module.exports = (job, jobsPath) => {
  const instance = instantiate(job, jobsPath);

  if (!instance) return;

  const priority = instance.priority || 0;
  const concurrency = instance.concurrency || 1;

  /**
   * Run the handle method for job instance. If job fails it
   * will run the failed method on the instance
   * @param  {string}   job
   * @param  {Function} done
   */
  const run = (job, done) => {
    co(function * () {
      yield instance.handle(...job.data);
      done();
    }).catch(error => {
      co(function * () {
        yield instance.failed(...job.data);
        done(error);
        debug(`Failed processing job ${job.type} with the error ${error.message}`);
      });
    });
  };

  return { run, priority, concurrency };
};
